@@ -38,6 +38,13 @@ websocket_api_key = click.option(
     required=not os.environ.get(Envs.WEBSOCKET_API_KEY)
 )
 
+force = click.option(
+    '--force',
+    is_flag=True,
+    help='Forces new REST requests for all missing data, even if that data has already been marked as missing',
+    required=False
+)
+
 
 @click.group(
     name='tickerdax',
@@ -78,6 +85,7 @@ def list_routes(**kwargs):
 
 @click.command()
 @config
+@force
 @rest_api_key
 def download(**kwargs):
     from tickerdax.downloader import Downloader
@@ -87,6 +95,7 @@ def download(**kwargs):
 
 @click.command()
 @config
+@force
 @websocket_api_key
 @email
 def stream(**kwargs):
