@@ -1,7 +1,10 @@
 import os
 import json
+import subprocess
 import sys
 import asyncio
+import time
+
 import requests
 import tempfile
 import logging
@@ -25,7 +28,8 @@ class TickerDax:
             websocket_api_key=None,
             fast_start=True,
             connect=True,
-            log_connection=True
+            log_connection=True,
+            force=False
     ):
         websocket_api_prefix = 'ws'
         rest_api_prefix = 'api'
@@ -38,9 +42,10 @@ class TickerDax:
         self.rest_values = []
         self.cached_values = []
         self.missing_values = []
-        self._batch_size = 100
+        self._batch_size = 500
         self._fast_start = fast_start
         self._log_connection = log_connection
+        self._force = force
 
         # rest api configuration
         self._rest_api_host = f'https://{rest_api_prefix}.{NAME}.com'
@@ -116,27 +121,39 @@ class TickerDax:
         Get the data from the cache that already exists, and which REST requests still need to be made.
 
         :param list[str] keys: A complete list of all the cache keys.
-        :returns: A tuple of the cache values and which REST requests still need to be made.
-        :rtype tuple(list, dict)
+        :returns: Which REST requests still need to be made.
+        :rtype dict
         """
         cache_values = self._redis_client.mget(keys)
 
         rest_requests = {}
         for key, cache_value in zip(keys, cache_values):
-            if not cache_value:
-                items = key.split('/')
-                symbol = items[-2]
-                timestamp = float(items[-1])
+            items = key.split('/')
+            symbol = items[-2]
+            timestamp = float(items[-1])
 
+            if not cache_value:
                 if not rest_requests.get(symbol):
                     rest_requests[symbol] = []
                 rest_requests[symbol].append(timestamp)
 
-        # load the json data from the cached values and filter out None values
-        cache_values = [json.loads(cache_value) for cache_value in cache_values if cache_value]
+            # if the data is nothing more than the symbol and id, then this should be
+            # marked as missing data.
+            elif json.loads(cache_value) == {'id': timestamp, 'symbol': symbol}:
+                # if force is true then request the already reported missing values
+                if self._force:
+                    if not rest_requests.get(symbol):
+                        rest_requests[symbol] = []
+                    rest_requests[symbol].append(timestamp)
+                else:
+                    self.missing_values.append(json.loads(cache_value))
 
-        # return the cached values and the needed rest requests
-        return cache_values, rest_requests
+            # otherwise, the data is already cached
+            else:
+                self.cached_values.append(json.loads(cache_value))
+
+        # return the needed rest requests
+        return rest_requests
 
     def _set_redis_client(self):
         """
@@ -167,7 +184,7 @@ class TickerDax:
         with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as stream:
             return stream.connect_ex((self._redis_server_address, port)) == 0
 
-    def _start_redis_server(self) -> None:
+    def _start_redis_server(self, attempts=0) -> None:
         """
         Starts the redis docker container.
         """
@@ -178,6 +195,18 @@ class TickerDax:
             else:
                 self._logger.warning(f'Failed to connect with fast start. Starting full reboot...')
 
+        # if in a linux environment, try starting redis manually and
+        if sys.platform.startswith('linux') and attempts < 1:
+            try:
+                subprocess.Popen('redis-server', shell=True)
+                self._logger.info('Attempting to start redis server on local system...')
+                time.sleep(5)
+                if self._set_redis_client():
+                    return
+                self._logger.warning('Failed to start redis server on local system...')
+            except Exception as error:
+                self._logger.warning('Failed to start redis server on local system...')
+
         # get a unused host port
         self._redis_host_port = self._get_unused_port_number(6379)
 
@@ -224,7 +253,6 @@ class TickerDax:
         :rtype list[dict]
         """
         result = []
-        rest_requests = []
         batch = []
         number_of_timestamp = len(timestamps)
         for index, key in enumerate(timestamps, 1):
@@ -232,7 +260,7 @@ class TickerDax:
 
             if index % self._batch_size == 0:
                 self._logger.info(f'batch requesting {index}/{number_of_timestamp} "{route}/{symbol}" timestamps...')
-                rest_requests.append(self._rest_request(route, symbol, batch))
+                result.extend(self._rest_request(route, symbol, batch))
 
                 # clear the batch
                 batch.clear()
@@ -242,11 +270,7 @@ class TickerDax:
             self._logger.info(
                 f'batch requesting {number_of_timestamp}/{number_of_timestamp} "{route}/{symbol}" timestamps...'
             )
-            rest_requests.append(self._rest_request(route, symbol, batch))
-
-        # gather the rest request batches concurrently
-        for response in await asyncio.gather(*rest_requests):
-            result.extend(response)
+            result.extend(self._rest_request(route, symbol, batch))
 
         self._logger.debug(f'batch "{route}/{symbol}" requests complete!')
         return result
@@ -276,10 +300,10 @@ class TickerDax:
                             self._logger.info(f'Cached: {pformat(data)}')
 
         except (ConnectionClosed, ConnectionClosedOK) as error:
-            self._logger.error(error)
-            self._logger.info('Trying to reconnect...')
-            # re-connect if the connection was closed
-            await self._stream_to_cache(route, symbols)
+            if error.status_code == 1001:
+                self._logger.info('refreshing connection...')
+                # re-connect if the connection was closed
+                await self._stream_to_cache(route, symbols)
 
         except InvalidStatusCode as error:
             if error.status_code == 401:
@@ -288,7 +312,8 @@ class TickerDax:
                     f'the {self._host} websocket API. Please check your credentials.'
                 )
 
-    async def _rest_request(self, route, symbol, timestamps):
+    # async def _rest_request(self, route, symbol, timestamps):
+    def _rest_request(self, route, symbol, timestamps):
         """
         Preforms a single REST request.
 
@@ -309,11 +334,19 @@ class TickerDax:
             if response.ok:
                 return response.json()
             else:
-                self._logger.error(response.text)
-                return []
+                if response.json().get('message') == 'Forbidden':
+                    self.report_error(
+                        'This API key is not authorized to connect to the https://api.tickerdax.com REST API. '
+                        'Please check your credentials.'
+                    )
+                elif response.json().get('message') == 'Limit Exceeded':
+                    self.report_error(
+                        'This API key has exceeded its usage limit. Go to https://tickerdax.com to upgrade your plan.'
+                    )
+                else:
+                    self.report_error(response.json())
         except Exception as error:
-            self._logger.error(error)
-            return []
+            self.report_error(str(error))
 
     async def _request(self, route, rest_requests):
         """
@@ -332,7 +365,6 @@ class TickerDax:
                 self._batch_request(route, symbol, timestamps) for symbol, timestamps in rest_requests.items()
             ]):
                 rest_values.extend(result)
-
         return rest_values
 
     async def _stream(self, routes):
@@ -341,8 +373,8 @@ class TickerDax:
 
         :param dict routes: A dictionary of route names and their symbols.
         """
-        await asyncio.gather(*[
-            self._stream_to_cache(f'/{self._format_route(route)}', ','.join(symbols)) for route, symbols in routes.items()
+        await asyncio.gather(*[self._stream_to_cache(
+                f'/{self._format_route(route)}', ','.join(symbols)) for route, symbols in routes.items()
         ])
 
     def _update_cache(self, route, keys, timeframe):
@@ -363,6 +395,7 @@ class TickerDax:
             key = f'{NAME}/{route}/{symbol}/{timestamp}'
             if key in keys:
                 keys.remove(key)
+
         result.extend(self.cached_values)
 
         # cache the rest values
@@ -420,10 +453,10 @@ class TickerDax:
         :returns: A list of all available routes from the REST api.
         :rtype: list
         """
-        ignored_routes = ['info/usage-plans']
+        ignored_routes = ['info/plans']
         routes = []
         for route in requests.get(f'{self._rest_api_host}/openapi.json').json().get('paths', {}).keys():
-            route = route.strip(f'/{self._rest_api_version}/')
+            route = route.replace(f'/{self._rest_api_version}/', '')
             if route not in ignored_routes:
                 routes.append(route)
         return routes
@@ -450,7 +483,7 @@ class TickerDax:
 
         # get the cached values and determine which rest requests are outstanding
         self._logger.debug(f'Checking "{route}" cache for {symbols}...')
-        self.cached_values, outstanding_rest_requests = self._get_from_cache(keys)
+        outstanding_rest_requests = self._get_from_cache(keys)
         self.rest_values = asyncio.run(self._request(route, outstanding_rest_requests))
 
         result = self._update_cache(route, keys, timeframe)
@@ -476,21 +509,21 @@ class TickerDax:
         except Exception as error:
             self._logger.error(error)
             self._logger.info('Trying to reconnect...')
-            asyncio.run(self._stream(routes))
+            self.stream(routes)
 
 
 if __name__ == '__main__':
-    client = TickerDax()
+    client = TickerDax(force=True)
 
     pprint(client.get_route(
-        route='into-the-block/predictions',
-        symbols=['BTC', 'LTC'],
+        route='predictions/ml-order-book-model-1',
+        symbols=['BTC'],
         start=datetime.now(tz=timezone.utc) - timedelta(hours=6),
         end=datetime.now(tz=timezone.utc)
     ))
 
-    client.stream(
-        routes={
-            'into-the-block/predictions': ['BTC', 'LTC'],
-        },
-    )
+    # client.stream(
+    #     routes={
+    #         'predictions/ml-order-book-model-1': ['BTC', 'LTC'],
+    #     },
+    # )
