@@ -43,7 +43,8 @@ class ConfigBase:
                              error=f'Your configs timeframe is not one of the supported timeframes {timeframes}'),
             'start': Or(str, datetime),
             Optional('end'): Or(str, datetime),
-            Optional('fill_to_now'): And(bool)
+            Optional('fill_to_now'): And(bool),
+            Optional('force'): And(bool)
         })
         schema.validate(self._config)
 
